function [ phi_out ] = fista(phi, psi, B, eta)

% This is an internal function used by the function 
% opt_measurements_mat
% This function implements the FISTA algorithm, described in the papers:
% A. Beck and M. Teboulle, "A fast iterative shrinkage-thresholding algorithm
% for linear inverse problems," SIAM Journal on Imaging Sciences, vol. 2, no. 1,
% pp. 183 - 202, March 2009.
% A. Beck and M. Teboulle, "Fast gradient-based algorithms for constrained
% total variation image denoising and deblurring problems," IEEE Transactions
% on Image Processing, vol. 18, no. 11, pp. 2419-2434, November 2009.

% 
% You are very welcome to use this code. For this, clearly acknowledge 
% the source of this code, and cite the paper that describes the algorithm:
% M. Mordechay and Y. Y. Schechner, "Matrix optimization for poisson compressed
% sensing," in Proceedings of IEEE GlobalSIP, December 2014, pp.684-688.
%
% (c) Copyright Moran Mordechay 2014. The matlab code is available for 
% non-commercial use and exploration.  For commercial use contact the
% author. The author is not liable for any damages or loss that might be 
% caused by use or connection to this code. 


p = size(phi,1);
n = size(phi,2);
t_old = 1;
K = 6;

s = mean(sum(phi));
a = 1/s;
phi = phi / s;
X_old = [phi(:); a];
Y = X_old;


for k = 1:K
   Y_mat = reshape(Y(1:p*n),p,n); 
   grad_mat = 2 * (Y_mat * psi - Y(p*n+1)*B) * psi'; 
   drv_a = sum(sum(2*B.*(Y(p*n+1)*B-Y_mat*psi)));
   grad = [grad_mat(:); drv_a];
   X_new = Y - eta * grad; 
%    negs = X_new < 0;
%    X_new(negs) = 0;
   for i = 1:n
     [ X_new((i-1)*p+1:(i-1)*p+p) ] = orthogonal_projection_on_delta_n( X_new((i-1)*p+1:(i-1)*p+p) );
   end 
   if (X_new(p*n+1) < 10^-30)
      X_new(p*n+1) = 10^-30; 
   end
   t_new = (1 + sqrt(1 + 4 * t_old ^ 2)) / 2;
   Y = X_new + ((t_old - 1) / t_new) * (X_new - X_old);
%    Y = X_new;
   negs = find(Y < 0);
   %Y(negs) = 0;
   if (numel(negs)>0)
     for i = 1:n
     [ Y((i-1)*p+1:(i-1)*p+p) ] = orthogonal_projection_on_delta_n( Y((i-1)*p+1:(i-1)*p+p) );
     end   
   end
   if (Y(p*n+1) < 10^-30)
      Y(p*n+1) = 10^-30; 
   end
   X_old = X_new;
   t_old = t_new;
end

phi_out = reshape(X_new(1:p*n),p,n)/X_new(p*n+1);

end

